@@ -1,4 +1,4 @@
-use anyhow::{bail, Error};
+use anyhow::{anyhow, bail, Error};
 use clap::{Args, Parser};
 use log::info;
 use odbc_api::{
@@ -96,6 +96,13 @@ struct QueryOpt {
     /// for its word regarding the maximum length of the columns.
     #[clap(long, short = 'm')]
     max_str_len: Option<usize>,
+    /// Setting a maximum string length does help with avoiding allocating really large buffers, and
+    /// save memory, but it may lead to values being truncated, if any fields are larger than the
+    /// number of bytes specified with `max-str-len`. By default a truncation will raise an error
+    /// and cause the tool to exit with a non-zero status code. Set this flag to continue execution
+    /// and just write the truncated values into the csv.
+    #[clap(long)]
+    ignore_truncation: bool,
     /// Path to the output csv file the returned values are going to be written to. If omitted the
     /// csv is going to be printed to standard out.
     #[clap(long, short = 'o')]
@@ -336,6 +343,7 @@ fn fetch(environment: &Environment, opt: FetchOpt) -> Result<(), Error> {
         connect_opts,
         batch_size,
         max_str_len,
+        ignore_truncation: false,
         output,
         query: query_str,
         parameters,
@@ -353,6 +361,7 @@ fn query(environment: &Environment, opt: &QueryOpt) -> Result<(), Error> {
         query,
         batch_size,
         max_str_len,
+        ignore_truncation,
     } = opt;
 
     // If an output file has been specified write to it, otherwise use stdout instead.
@@ -377,7 +386,13 @@ fn query(environment: &Environment, opt: &QueryOpt) -> Result<(), Error> {
     match connection.execute(query, params.as_slice())? {
         Some(cursor) => {
             // Write column names.
-            cursor_to_csv(cursor, &mut writer, *batch_size, *max_str_len)?;
+            cursor_to_csv(
+                cursor,
+                &mut writer,
+                *batch_size,
+                *max_str_len,
+                *ignore_truncation,
+            )?;
         }
         None => {
             eprintln!("Query came back empty (not even a schema has been returned). No output has been created.");
@@ -495,7 +510,7 @@ fn tables(environment: &Environment, table_opt: &ListTablesOpt) -> Result<(), Er
     // Limit the amount of memory allocated for each column element in case some driver decides to
     // report crazy max column lengths (e.g. MariaDB on Windows)
     let max_str_len = Some(4096);
-    cursor_to_csv(cursor, &mut writer, 100, max_str_len)?;
+    cursor_to_csv(cursor, &mut writer, 100, max_str_len, true)?;
     Ok(())
 }
 
@@ -523,7 +538,7 @@ fn columns(environment: &Environment, columns_opt: &ListColumnsOpt) -> Result<()
     // Limit the amount of memory allocated for each column element in case some driver decides to
     // report crazy max column lengths (e.g. MariaDB on Windows)
     let max_str_len = Some(4096);
-    cursor_to_csv(cursor, &mut writer, 100, max_str_len)?;
+    cursor_to_csv(cursor, &mut writer, 100, max_str_len, true)?;
     Ok(())
 }
 
@@ -532,13 +547,17 @@ fn cursor_to_csv(
     writer: &mut csv::Writer<impl Write>,
     batch_size: usize,
     max_str_len: Option<usize>,
+    ignore_truncation: bool,
 ) -> Result<(), Error> {
     let headline: Vec<String> = cursor.column_names()?.collect::<Result<_, _>>()?;
     writer.write_record(headline)?;
     let mut buffers = TextRowSet::for_cursor(batch_size, &cursor, max_str_len)?;
     let mut row_set_cursor = cursor.bind_buffer(&mut buffers)?;
     let mut num_batch = 0;
-    while let Some(buffer) = row_set_cursor.fetch()? {
+    while let Some(buffer) = row_set_cursor
+        .fetch_with_truncation_check(!ignore_truncation)
+        .map_err(provide_context_for_truncation_error)?
+    {
         num_batch += 1;
         info!(
             "Fetched batch {} with {} rows.",
@@ -553,3 +572,18 @@ fn cursor_to_csv(
     }
     Ok(())
 }
+
+fn provide_context_for_truncation_error(error: odbc_api::Error) -> Error {
+    match error {
+        odbc_api::Error::TooLargeValueForBuffer => {
+            anyhow!(
+                "Truncation of text or binary data detected. Try using larger values of \
+                `--max-str-len` (or do not specify it at all) in order to allow for larger values.
+                You can also use the `--ignore-truncation` flag in order to consider truncations
+                warnings only. This will cause the truncated value to be written into the csv, and
+                execution to be continued normally."
+            )
+        }
+        other => other.into(),
+    }
+}
